using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Xunit;
using Amazon.Lambda.Core;
using Amazon.Lambda.TestUtilities;

using CreateCommand = AWS.APG.CQRSES.CommandCreateLambda;
using DeleteCommand = AWS.APG.CQRSES.CommandDeleteLambda;
using UpdateCommand = AWS.APG.CQRSES.CommandUpdateLambda;
using AddRewardCommand = AWS.APG.CQRSES.CommandAddRewardLambda;
using RedeemRewardCommand = AWS.APG.CQRSES.CommandRedeemRewardLambda;

namespace AWS.APG.CQRSES.CQRSLambda.Tests
{
    public class CommandFunctionTest : BaseFunctionTest
    {


        [Fact]
        public async Task CreateUpdateDeleteTestFunction()
        {
            
            var createCommand = new CreateCommand.Function(AccessKey, SecretKey);
            var context = new TestLambdaContext() 
            { 
                FunctionName = "CommandFunctionHandlerTest"
            };

            int id = 5000;

            try
            { 
                await createCommand.FunctionHandler(
                    new CreateCommand.Models.Customer()
                    {
                        Id = id,
                        Address = "Texas USA",
                        CompanyName = "Blue Origin",
                        Firstname = "John",
                        Lastname = "Doe",
                        VIP = true
                    }, 
                    context);

                var updateCommand = new UpdateCommand.Function(AccessKey, SecretKey);
                await updateCommand.FunctionHandler(
                    new UpdateCommand.Models.Customer()
                    {
                        Id = id,
                        Address = "Seatle USA",
                        CompanyName = "Amazon Web Service",
                        Firstname = "John",
                        Lastname = "Wick",
                        VIP = false
                    }, 
                    context);

                var deleteCommand = new DeleteCommand.Function(AccessKey, SecretKey);
                await deleteCommand.FunctionHandler(id, context);

            }

            catch (Exception ex)
            {
                Assert.False(true, ex.Message);
            }
        }

        /// <summary>
        /// Add reward command.
        /// </summary>
        /// <returns></returns>
        [Fact]
        public async Task CustomerAddRewardTestFunction()
        {
            var command = new AddRewardCommand.Function(AccessKey, SecretKey);
            var context = new TestLambdaContext()
            {
                FunctionName = "AddRewardCommandFunctionHandlerTest"
            };

            try
            {
                await command.FunctionHandler(
                    new AddRewardCommand.Models.Reward()
                    {
                        Id = 20001,
                        CustomerId = 1,
                        Points = 150,
                        Timestamp = DateTime.Now
                    },
                    context);

                await command.FunctionHandler(
                   new AddRewardCommand.Models.Reward()
                   {
                       Id = 20002,
                       CustomerId = 2,
                       Points = 55,
                       Timestamp = DateTime.Now
                   },
                   context);

            }

            catch(Exception ex)
            {
                Assert.False(true, ex.Message);
            }
            
        }


        /// <summary>
        /// Test the redeem reward command.
        /// </summary>
        /// <returns></returns>
        [Fact]
        public async Task CustomerRedeemRewardTestFunction()
        {
            var command = new RedeemRewardCommand.Function(AccessKey, SecretKey);
            var context = new TestLambdaContext()
            {
                FunctionName = "RedeemCommandFunctionHandlerTest"
            };

            try
            {
                await command.FunctionHandler(
                    new RedeemRewardCommand.Models.Reward()
                    {
                        Id = 20003,
                        CustomerId = 1,
                        Points = 10,
                        Timestamp = DateTime.Now
                    },
                    context);

                await command.FunctionHandler(
                   new RedeemRewardCommand.Models.Reward()
                   {
                       Id = 20004,
                       CustomerId = 2,
                       Points = 5,
                       Timestamp = DateTime.Now
                   },
                   context);
            }

            catch (Exception ex)
            {
                Assert.False(true, ex.Message);
            }

        }



        
    }
}
