﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Xunit;
using Amazon.Lambda.Core;
using Amazon.Lambda.TestUtilities;

using CustomerListQuery = AWS.APG.CQRSES.QueryCustomerListLambda;
using CustomerRewardQuery = AWS.APG.CQRSES.QueryRewardLambda;

namespace AWS.APG.CQRSES.CQRSLambda.Tests
{
    public class QueryFunctionTest : BaseFunctionTest
    {
        [Fact]
        public async Task CustomerListTestFunction()
        {
            var query = new CustomerListQuery.Function(AccessKey, SecretKey);
            var context = new TestLambdaContext()
            {
                FunctionName = "QueryFunctionHandlerTest"
            };

            var result = await query.FunctionHandler(context);

            Assert.NotNull(result);
            Assert.True(result.Count > 0, "Should have at least one record.");
        }


        [Fact]
        public async Task CustomerRewardTotalPointsTestFunction()
        {
            var query = new CustomerRewardQuery.Function(AccessKey, SecretKey);
            var context = new TestLambdaContext()
            {
                FunctionName = "QueryCustomerRewardFunctionHandlerTest"
            };

            //-- Richard Roe customer
            var result = await query.FunctionHandler(1, context);

            Assert.NotNull(result);
            Assert.Equal("Richard Roe", result.CustomerName);
            Assert.True(result.Points > 0);
        }
    }
}
