using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.Lambda.Core;
using Amazon.Runtime;
using AWS.APG.CQRSES.CommandUpdateLambda.Models;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace AWS.APG.CQRSES.CommandUpdateLambda
{
    public class Function
    {

        private static AmazonDynamoDBClient _client;
        private readonly string _accessKey;
        private readonly string _secretKey;


        /// <summary>
        /// For testing use only.
        /// </summary>
        public Function(string accessKey,
            string secretKey)
        {
            _accessKey = accessKey;
            _secretKey = secretKey;

            Initialize();
        }

        public Function()
        {
            //-- get keys and service url from enviornment variable
            _accessKey = Environment.GetEnvironmentVariable("AccessKey");
            _secretKey = Environment.GetEnvironmentVariable("SecretKey");

            Initialize();
        }

        private void Initialize()
        {
            _client = new AmazonDynamoDBClient(
                new BasicAWSCredentials(_accessKey, _secretKey),
                new AmazonDynamoDBConfig
                {
                    //ServiceURL = _serviceUrl,
                    RegionEndpoint = RegionEndpoint.USEast1
                });
        }


        /// <summary>
        /// A simple function that takes a string and does a ToUpper
        /// </summary>
        /// <param name="input"></param>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task FunctionHandler(Customer input, ILambdaContext context)
        {
            //When run in AWS this will be logged in Cloud Watch
            Console.WriteLine("\nStarted executing Function {0}", context.FunctionName);

            DynamoDBContext dbContext = new DynamoDBContext(_client);

            //get the record
            Customer customer = await dbContext.LoadAsync<Customer>(input.Id);

            if (customer != null)
            {
                //update the columns
                customer.Address = input.Address;
                customer.CompanyName = input.CompanyName;
                customer.Firstname = input.Firstname;
                customer.Lastname = input.Lastname;
                customer.VIP = input.VIP;

                //Update the record
                await dbContext.SaveAsync<Customer>(customer);

                Console.WriteLine("\nUpdated record {0}", customer.Id);
            }

            //When run in AWS this will be logged in Cloud Watch
            Console.WriteLine("\nFinish executing Function {0} @ {1}", context.FunctionName, DateTime.Now);
        }
    }
}
