﻿using AWS.APG.CQRSES.Common;
using AWS.APG.CQRSES.Common.Data;
using System;
using System.Collections.Generic;
using System.Linq;

namespace AWS.APG.CQRSES.DAL
{
    /// <summary>
    /// Customer data access object that contains CRUD interfaces.
    /// </summary>
    public class CustomerDAO
    {

        #region CRUD interfaces

        private int _customerId = 1000;
        private static CustomerData _data;

        public CustomerDAO()
        {
            if(_data == null)
                _data = new CustomerData();
        }

        
        public int Create(CustomerEntity entity)
        {
            _customerId += 1;
            //insert code here
            entity.Id = _customerId;

            _data.Customers().Add(entity);

            return _customerId;
        }

        public void Update(CustomerEntity entity)
        {
            var customer = _data.Customers().Find(c => c.Id == entity.Id);


            //update code here
            if (customer != null)
            {
                customer.CompanyName = entity.CompanyName;
                customer.Address = entity.Address;
                customer.Firstname = entity.Firstname;
                customer.Lastname = entity.Lastname;
                customer.VIP = entity.VIP;
            }
        }

        public void Delete(int customerId)
        {
            var customer = _data.Customers().Find(c => c.Id == customerId);

            //Delete code here
            if (customer != null)
            {
                _data.Customers().Remove(customer);
            }
        }

        public int AddPoints(int customerId, int points)
        {
            int rewardId = Helper.GetRewardId();

            //Add points here
            _data.CustomerRewards().Add(new CustomerRewardPointEntity()
            {
                Id = rewardId,
                CustomerId = customerId,
                Points = points,
                Timestamp = DateTime.Now
            });

            return rewardId;
        }

        public int RedeemPoints(int customerId, int points)
        {
            int rewardId = Helper.GetRewardId();

            //Redeem points
            _data.CustomerRewards().Add(new CustomerRewardPointEntity()
            {
                Id = rewardId,
                CustomerId = customerId,
                Points = -1 * points,
                Timestamp = DateTime.Now
            });

            return rewardId;
        }

        public CustomerEntity GetCustomer(int customerId)
        {
            return _data.Customers()
                .SingleOrDefault(c => c.Id == customerId);
        }

        public List<CustomerEntityList> GetVIPCustomers()
        {
            //Select statement goes here

            return new List<CustomerEntityList>(_data.CustomerList()
                .Where(c => c.VIP == true));
        }

        public IList<CustomerEntityList> GetCustomerList()
        {
            //Select statement goes here

            return _data.CustomerList();
        }

        public int GetCustomerPoints(int customerId)
        {
            int points = _data.CustomerRewards()
                .Where(c => c.CustomerId == customerId)
                .Sum(c => c.Points);

            //Select statement goes here

            return points;
        }


        public IList<object> GetMonthlyStatement(int customerId,
            int month,
            int year)
        {
            int points = 0;

            //Select statement goes here

            return new List<object>();
        }

        #endregion
    }

}
