using System;
using System.IO;
using System.Text;

using Amazon.Lambda.Core;
using Amazon.DynamoDBv2.Model;
using AWS.APG.CQRSES.EventSourceReward.Models;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.Lambda.DynamoDBEvents;
using Amazon.Runtime;
using System.Threading.Tasks;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace AWS.APG.CQRSES.EventSourceReward
{
    public class Function
    {
        private static AmazonDynamoDBClient _client;
        private readonly string _accessKey;
        private readonly string _secretKey;


        /// <summary>
        /// For testing use only.
        /// </summary>
        public Function(string accessKey,
            string secretKey)
        {
            _accessKey = accessKey;
            _secretKey = secretKey;

            Initialize();
        }

        public Function()
        {
            //-- get keys and service url from enviornment variable
            _accessKey = Environment.GetEnvironmentVariable("AccessKey");
            _secretKey = Environment.GetEnvironmentVariable("SecretKey");

            Initialize();
        }

        private void Initialize()
        {
            _client = new AmazonDynamoDBClient(
                new BasicAWSCredentials(_accessKey, _secretKey),
                new AmazonDynamoDBConfig
                {
                    //ServiceURL = _serviceUrl,
                    RegionEndpoint = RegionEndpoint.USEast1
                });
        }

        public async Task FunctionHandler(DynamoDBEvent dynamoEvent, ILambdaContext context)
        {
            context.Logger.LogLine($"Beginning to process {dynamoEvent.Records.Count} records...");

            await UpdateAggregatedReward(dynamoEvent, context);

            context.Logger.LogLine("Stream processing complete.");
        }


        private static async Task UpdateAggregatedReward(DynamoDBEvent dynamoEvent, ILambdaContext context)
        {
            DynamoDBContext dbContext = new DynamoDBContext(_client);            

            foreach (var record in dynamoEvent.Records)
            {
                context.Logger.LogLine($"Event ID: {record.EventID}");
                context.Logger.LogLine($"Event Name: {record.EventName}");

                if (record.EventName == "INSERT")
                {
                    //-- get the newly added reward from the stream
                    var reward = new Reward(record.Dynamodb.NewImage);

                    //-- get the customer reward from aggregated customer reward table
                    var customerReward = await dbContext.LoadAsync<CustomerReward>(reward.CustomerId);
                    //-- add the reward points
                    customerReward.Points += reward.Points;

                    await dbContext.SaveAsync<CustomerReward>(customerReward);
                }                
            }
        }
    }
}