using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.Lambda.Core;
using Amazon.Runtime;
using AwS.APG.CQRSES.QueryRewardLambda.Models;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace AWS.APG.CQRSES.QueryRewardLambda
{
    public class Function
    {

        private static AmazonDynamoDBClient _client;
        private readonly string _accessKey;
        private readonly string _secretKey;


        /// <summary>
        /// For testing use only.
        /// </summary>
        public Function(string accessKey,
            string secretKey)
        {
            _accessKey = accessKey;
            _secretKey = secretKey;

            Initialize();
        }

        public Function()
        {
            //-- get keys and service url from enviornment variable
            _accessKey = Environment.GetEnvironmentVariable("AccessKey");
            _secretKey = Environment.GetEnvironmentVariable("SecretKey");

            Initialize();
        }

        private void Initialize()
        {
            _client = new AmazonDynamoDBClient(
                new BasicAWSCredentials(_accessKey, _secretKey),
                new AmazonDynamoDBConfig
                {
                    //ServiceURL = _serviceUrl,
                    RegionEndpoint = RegionEndpoint.USEast1
                });
        }


        /// <summary>
        /// A simple function that takes a string and does a ToUpper
        /// </summary>
        /// <param name="input"></param>
        /// <param name="context"></param>
        /// <returns></returns>
        public async Task<CustomerReward> FunctionHandler(int id, ILambdaContext context)
        {
            //When run in AWS this will be logged in Cloud Watch
            Console.WriteLine("Started executing Function {0}", context.FunctionName);

            DynamoDBContext dbContext = new DynamoDBContext(_client);
            var customerReward = await dbContext
                .LoadAsync<CustomerReward>(id);

            //When run in AWS this will be logged in Cloud Watch
            Console.WriteLine("Finish executing Function {0} @ {1}",
                context.FunctionName,
                DateTime.Now);

            return customerReward;
        }
    }
}
