﻿using System;
using System.Collections.Generic;
using Amazon;
using Amazon.DynamoDBv2;
using Amazon.DynamoDBv2.DataModel;
using Amazon.DynamoDBv2.Model;
using Amazon.Runtime;
using AwS.APG.CQRSES.Build.Models;

namespace AwS.APG.CQRSES.Build
{
    class Program
    {
        private static AmazonDynamoDBClient client;

        static Program()
        {
            AmazonDynamoDBConfig config = new AmazonDynamoDBConfig();
            config.RegionEndpoint = RegionEndpoint.USEast1;

            //NOTE: Change this with your Access key and Secret key that has Dynamo DB permission to create and do CRUD operations to table
            //      This is for DEMO ONLY, secret and access keys SHOULD NOT BE HARDCODED IN THE CODE. 
            //      We need this to build the tables and populate it with demo data.
            AWSCredentials credentials = new BasicAWSCredentials("YOUR ACCESS KEY", "YOUR SECRET KEY");
            
            client = new AmazonDynamoDBClient(credentials, config);
        }

        static void Main(string[] args)
        {
            try
            {
                /*
                This a build script that will create demo dynamodb tables and populate it with demo data.
                Before running this script make sure credentials is supplied above and Clean-up section is commented.  
                 */


                #region > Create Tables

                CreateCustomerCommandTable();
                GetTableInformation("cqrses-customer-cmd");

                CreateCustomerListQueryTable();
                GetTableInformation("cqrses-customer-query");

                CreateRewardCommandTable();
                GetTableInformation("cqrses-reward-cmd");

                CreateCustomerRewardQueryTable();
                GetTableInformation("cqrses-reward-query");

                ListMyTables();
                
                #endregion


                #region > Seed Data. Populatest the table with demo data.

                //Populates Customer table in command table
                SeedCustomer();

                //Populates Customer list table in query table
                SeedCustomerList();

                //Populate customer rewards
                SeedCustomerRewards();

                //Populate Customer aggregated rewards
                SeedCustomerTotalRewards();

                #endregion


                #region > Clean Up. Uncomment this to drop all the tables. Before running comment all code found in Create Table and Seed Data region above.

                //DeleteExampleTable("cqrses-customer-cmd");
                //DeleteExampleTable("cqrses-customer-query");
                //DeleteExampleTable("cqrses-reward-cmd");
                //DeleteExampleTable("cqrses-reward-query");

                #endregion


                Console.WriteLine("To continue, press Enter");
                Console.ReadLine();
            }
            catch (AmazonDynamoDBException e) { Console.WriteLine(e.Message); }
            catch (AmazonServiceException e) { Console.WriteLine(e.Message); }
            catch (Exception e) { Console.WriteLine(e.Message); }
        }

        private static void CreateCustomerCommandTable()
        {
            string tableName = "cqrses-customer-cmd";
            Console.WriteLine("\n*** Creating table " + tableName +" ***");

            //Stream specification
            StreamSpecification streamSpec = new StreamSpecification 
            { 
                StreamEnabled = true,
                StreamViewType = "NEW_AND_OLD_IMAGES"
            };


            var request = new CreateTableRequest
            {
                AttributeDefinitions = new List<AttributeDefinition>()
                {
                    new AttributeDefinition
                    {
                        AttributeName = "Id",
                        AttributeType = "N"
                    }
                },
                KeySchema = new List<KeySchemaElement>
                {
                    new KeySchemaElement
                    {
                        AttributeName = "Id",
                        KeyType = "HASH" //Partition key
                    }
                },
                ProvisionedThroughput = new ProvisionedThroughput
                {
                    ReadCapacityUnits = 5,
                    WriteCapacityUnits = 6
                },

                TableName = tableName,
                StreamSpecification = streamSpec
            };

            var response = client.CreateTableAsync(request).Result;

            var tableDescription = response.TableDescription;
            Console.WriteLine("{1}: {0} \t ReadsPerSec: {2} \t WritesPerSec: {3} \t StreamArn: {4}",
                      tableDescription.TableStatus,
                      tableDescription.TableName,
                      tableDescription.ProvisionedThroughput.ReadCapacityUnits,
                      tableDescription.ProvisionedThroughput.WriteCapacityUnits,
                      response.TableDescription.LatestStreamArn);

            string status = tableDescription.TableStatus;
            Console.WriteLine(tableName + " - " + status);

            WaitUntilTableReady(tableName);
        }

        private static void CreateCustomerListQueryTable()
        {
            string tableName = "cqrses-customer-query";
            Console.WriteLine("\n*** Creating table " + tableName + " ***");

            var request = new CreateTableRequest
            {
                AttributeDefinitions = new List<AttributeDefinition>()
                {
                    new AttributeDefinition
                    {
                        AttributeName = "Id",
                        AttributeType = "N"
                    }
                },
                KeySchema = new List<KeySchemaElement>
                {
                    new KeySchemaElement
                    {
                        AttributeName = "Id",
                        KeyType = "HASH" //Partition key
                    }
                },
                ProvisionedThroughput = new ProvisionedThroughput
                {
                    ReadCapacityUnits = 5,
                    WriteCapacityUnits = 6
                },

                TableName = tableName
            };

            var response = client.CreateTableAsync(request).Result;

            var tableDescription = response.TableDescription;
            Console.WriteLine("{1}: {0} \t ReadsPerSec: {2} \t WritesPerSec: {3}",
                      tableDescription.TableStatus,
                      tableDescription.TableName,
                      tableDescription.ProvisionedThroughput.ReadCapacityUnits,
                      tableDescription.ProvisionedThroughput.WriteCapacityUnits);

            //response.TableDescription.LatestStreamArn

            string status = tableDescription.TableStatus;
            Console.WriteLine(tableName + " - " + status);

            WaitUntilTableReady(tableName);
        }

        private static void CreateRewardCommandTable()
        {
            string tableName = "cqrses-reward-cmd";
            Console.WriteLine("\n*** Creating table " + tableName + " ***");

            //Stream specification
            StreamSpecification streamSpec = new StreamSpecification
            {
                StreamEnabled = true,
                StreamViewType = "NEW_IMAGE"
            };


            var request = new CreateTableRequest
            {
                AttributeDefinitions = new List<AttributeDefinition>()
                {
                    new AttributeDefinition
                    {
                        AttributeName = "Id",
                        AttributeType = "N"
                    },
                    new AttributeDefinition
                    {
                        AttributeName = "Timestamp",
                        AttributeType = "S"
                    }
                },
                KeySchema = new List<KeySchemaElement>
                {
                    new KeySchemaElement
                    {
                        AttributeName = "Id",
                        KeyType = "HASH" //Partition key
                    },
                    new KeySchemaElement
                    {
                        AttributeName = "Timestamp",
                        KeyType = "RANGE"
                    }
                },
                ProvisionedThroughput = new ProvisionedThroughput
                {
                    ReadCapacityUnits = 5,
                    WriteCapacityUnits = 6
                },

                TableName = tableName,
                StreamSpecification = streamSpec
            };

            var response = client.CreateTableAsync(request).Result;

            var tableDescription = response.TableDescription;
            Console.WriteLine("{1}: {0} \t ReadsPerSec: {2} \t WritesPerSec: {3} \t StreamArn: {4}",
                      tableDescription.TableStatus,
                      tableDescription.TableName,
                      tableDescription.ProvisionedThroughput.ReadCapacityUnits,
                      tableDescription.ProvisionedThroughput.WriteCapacityUnits,
                      response.TableDescription.LatestStreamArn);

            string status = tableDescription.TableStatus;
            Console.WriteLine(tableName + " - " + status);

            WaitUntilTableReady(tableName);
        }

        private static void CreateCustomerRewardQueryTable()
        {
            string tableName = "cqrses-reward-query";
            Console.WriteLine("\n*** Creating table " + tableName + " ***");

            var request = new CreateTableRequest
            {
                AttributeDefinitions = new List<AttributeDefinition>()
                {
                    new AttributeDefinition
                    {
                        AttributeName = "CustomerId",
                        AttributeType = "N"
                    }
                },
                KeySchema = new List<KeySchemaElement>
                {
                    new KeySchemaElement
                    {
                        AttributeName = "CustomerId",
                        KeyType = "HASH" //Partition key
                    }
                },
                ProvisionedThroughput = new ProvisionedThroughput
                {
                    ReadCapacityUnits = 5,
                    WriteCapacityUnits = 6
                },

                TableName = tableName
            };

            var response = client.CreateTableAsync(request).Result;

            var tableDescription = response.TableDescription;
            Console.WriteLine("{1}: {0} \t ReadsPerSec: {2} \t WritesPerSec: {3}",
                      tableDescription.TableStatus,
                      tableDescription.TableName,
                      tableDescription.ProvisionedThroughput.ReadCapacityUnits,
                      tableDescription.ProvisionedThroughput.WriteCapacityUnits);

            //response.TableDescription.LatestStreamArn

            string status = tableDescription.TableStatus;
            Console.WriteLine(tableName + " - " + status);

            WaitUntilTableReady(tableName);
        }

        private static void ListMyTables()
        {
            Console.WriteLine("\n*** listing tables ***");
            string lastTableNameEvaluated = null;
            do
            {
                var request = new ListTablesRequest
                {
                    Limit = 2,
                    ExclusiveStartTableName = lastTableNameEvaluated
                };

                var response = client.ListTablesAsync(request).Result;
                foreach (string name in response.TableNames)
                    Console.WriteLine(name);

                lastTableNameEvaluated = response.LastEvaluatedTableName;
            } while (lastTableNameEvaluated != null);
        }

        private static void GetTableInformation(string tableName)
        {
            Console.WriteLine("\n*** Retrieving table information ***");
            var request = new DescribeTableRequest
            {
                TableName = tableName
            };

            var response = client.DescribeTableAsync(request).Result;

            TableDescription description = response.Table;
            Console.WriteLine("Name: {0}", description.TableName);
            Console.WriteLine("# of items: {0}", description.ItemCount);
            Console.WriteLine("Provision Throughput (reads/sec): {0}",
                      description.ProvisionedThroughput.ReadCapacityUnits);
            Console.WriteLine("Provision Throughput (writes/sec): {0}",
                      description.ProvisionedThroughput.WriteCapacityUnits);
        }

        private static void UpdateExampleTable(string tableName)
        {
            Console.WriteLine("\n*** Updating table ***");
            var request = new UpdateTableRequest()
            {
                TableName = tableName,
                ProvisionedThroughput = new ProvisionedThroughput()
                {
                    ReadCapacityUnits = 6,
                    WriteCapacityUnits = 7
                }
            };

            var response = client.UpdateTableAsync(request).Result;

            WaitUntilTableReady(tableName);
        }

        private static void DeleteExampleTable(string tableName)
        {
            Console.WriteLine("\n*** Deleting table ***");
            var request = new DeleteTableRequest
            {
                TableName = tableName
            };

            var response = client.DeleteTableAsync(request).Result;

            Console.WriteLine("Table is being deleted...");
        }

        private static void WaitUntilTableReady(string tableName)
        {
            string status = null;
            // Let us wait until table is created. Call DescribeTable.
            do
            {
                System.Threading.Thread.Sleep(5000); // Wait 5 seconds.
                try
                {
                    var res = client.DescribeTableAsync(new DescribeTableRequest
                    {
                        TableName = tableName
                    }).Result;

                    Console.WriteLine("Table name: {0}, status: {1}",
                              res.Table.TableName,
                              res.Table.TableStatus);
                    status = res.Table.TableStatus;
                }
                catch (ResourceNotFoundException)
                {
                    // DescribeTable is eventually consistent. So you might
                    // get resource not found. So we handle the potential exception.
                }
            } while (status != "ACTIVE");
        }

        private static void SeedCustomer()
        {
            DynamoDBContext context = new DynamoDBContext(client);
            var batch = context.CreateBatchWrite<Customer>();
            
            Console.WriteLine("\n*** Adding customers... ***");

            batch.AddPutItems(new List<Customer>()
            {
                new Customer(){ Id = 1, Address="Metro Manila, Philippines", CompanyName="ABC Company", Firstname="Richard", Lastname="Roe", VIP= false },
                new Customer(){ Id = 2, Address="Sydney, Australia", CompanyName="LMN Company", Firstname="Arnav", Lastname="Desai", VIP= true },
                new Customer(){ Id = 3, Address="Kuala Lumpur, Malaysia", CompanyName="XYZ Company", Firstname="Shirley", Lastname="Rodriquez", VIP= true }
            });


            batch.ExecuteAsync().Wait();

            Console.WriteLine("\nPopulated 3 customers...");
        }

        private static void SeedCustomerRewards()
        {
            DynamoDBContext context = new DynamoDBContext(client);
            var batch = context.CreateBatchWrite<Reward>();

            Console.WriteLine("\n*** Adding customer rewards... ***");

            batch.AddPutItems(new List<Reward>()
            {
                new Reward()
                {
                    Id = 10001,
                    CustomerId = 1,
                    Points = 25,
                    Timestamp = DateTime.Now
                },

                new Reward()
                {
                    Id = 10002,
                    CustomerId = 1,
                    Points = 80,
                    Timestamp = DateTime.Now
                },

                new Reward()
                {
                    Id = 10003,
                    CustomerId = 2,
                    Points = 5,
                    Timestamp = DateTime.Now
                },

                new Reward()
                {
                    Id = 10004,
                    CustomerId = 2,
                    Points = 96,
                    Timestamp = DateTime.Now
                },

                new Reward()
                {
                    Id = 10005,
                    CustomerId = 3,
                    Points = 30,
                    Timestamp = DateTime.Now
                },

                new Reward()
                {
                    Id = 10006,
                    CustomerId = 3,
                    Points = 70,
                    Timestamp = DateTime.Now
                }
            });


            batch.ExecuteAsync().Wait();

            Console.WriteLine("\nPopulated customer rewards...");
        }

        private static void SeedCustomerTotalRewards()
        {
            DynamoDBContext context = new DynamoDBContext(client);
            var batch = context.CreateBatchWrite<CustomerReward>();

            Console.WriteLine("\n*** Adding customer rewards... ***");

            batch.AddPutItems(new List<CustomerReward>()
            {
                new CustomerReward()
                {
                    CustomerId = 1,
                    Points = 50,
                    CustomerName = "Richard Roe"
                },

                new CustomerReward()
                {
                    CustomerId = 2,
                    Points = 200,
                    CustomerName = "Arnav Desai"
                },

                new CustomerReward()
                {
                    CustomerId = 3,
                    Points = 70,
                    CustomerName = "Shirley Rodriquez"
                }
            });


            batch.ExecuteAsync().Wait();

            Console.WriteLine("\nPopulated customer aggregated rewards...");
        }

        private static void SeedCustomerList()
        {
            DynamoDBContext context = new DynamoDBContext(client);
            var batch = context.CreateBatchWrite<CustomerList>();

            Console.WriteLine("\n*** Adding customers list... ***");

            batch.AddPutItems(new List<CustomerList>()
            {
                new CustomerList(){ Id = 1, Firstname="Richard", Lastname="Roe", VIP= false },
                new CustomerList(){ Id = 2, Firstname="Arnav", Lastname="Desai", VIP= true },
                new CustomerList(){ Id = 3, Firstname="Shirley", Lastname="Rodriquez", VIP= true }
            });

            Console.WriteLine("\nCustomers list populated...");

            batch.ExecuteAsync().Wait();
        }
    }
}
